@@ -1,6 +1,6 @@
 #include "config.h"
 
-#include "xyzsh/xyzsh.h"
+#include "xyzsh.h"
 #include <string.h>
 #include <stdio.h>
 #include <glob.h>
@@ -92,6 +92,7 @@ BOOL sRunInfo_put_option(sRunInfo* self, MANAGED char* key)
         if(p->mKey) {
             p++;
             if(p == self->mOptions + hash_value) {
+                FREE(key);
                 return FALSE;
             }
             else if(p == self->mOptions + XYZSH_OPTION_MAX) {
@@ -141,6 +142,8 @@ BOOL sRunInfo_put_option_with_argument(sRunInfo* self, MANAGED char* key, MANAGE
         if(p->mKey) {
             p++;
             if(p == self->mOptions + hash_value) {
+                FREE(key);
+                FREE(argument);
                 return FALSE;
             }
             else if(p == self->mOptions + XYZSH_OPTION_MAX) {
@@ -246,8 +249,6 @@ void add_str_to_buf(sBuf* buf, char* str, int len)
 
 static void sCommand_copy_deeply(sCommand* dest, sCommand* src)
 {
-    dest->mKind = src->mKind;
-
     dest->mArgs = MALLOC(sizeof(char*)*src->mArgsSize);
     dest->mArgsFlags = MALLOC(sizeof(int)*src->mArgsSize);
     dest->mArgsNum = src->mArgsNum;
@@ -308,8 +309,6 @@ static void sCommand_copy_deeply(sCommand* dest, sCommand* src)
 
 static void sCommand_copy_deeply_stack(sCommand* dest, sCommand* src)
 {
-    dest->mKind = src->mKind;
-
     dest->mArgs = MALLOC(sizeof(char*)*src->mArgsSize);
     dest->mArgsFlags = MALLOC(sizeof(int)*src->mArgsSize);
     dest->mArgsNum = src->mArgsNum;
@@ -403,7 +402,7 @@ void sCommand_delete(sCommand* self)
     }
 }
 
-BOOL sCommand_add_arg(sCommand* self, MANAGED char* buf, int env, int quoted_string, int glob, int option, char* sname, int sline)
+BOOL sCommand_add_arg(sCommand* self, MANAGED char* buf, int env, int quoted_string, int quoted_head, int glob, int option, char* sname, int sline)
 {
     if(self->mArgsNum+1 >= self->mArgsSize) {
         const int arg_size = self->mArgsSize;
@@ -424,7 +423,7 @@ BOOL sCommand_add_arg(sCommand* self, MANAGED char* buf, int env, int quoted_str
     }
 
     self->mArgs[self->mArgsNum] = buf;
-    self->mArgsFlags[self->mArgsNum++] = (env ? XYZSH_ARGUMENT_ENV:0) | (glob ? XYZSH_ARGUMENT_GLOB:0) | (option ? XYZSH_ARGUMENT_OPTION : 0) | (quoted_string ? XYZSH_ARGUMENT_QUOTED:0);
+    self->mArgsFlags[self->mArgsNum++] = (env ? XYZSH_ARGUMENT_ENV:0) | (glob ? XYZSH_ARGUMENT_GLOB:0) | (option ? XYZSH_ARGUMENT_OPTION : 0) | (quoted_string ? XYZSH_ARGUMENT_QUOTED:0) | (quoted_head ? XYZSH_ARGUMENT_QUOTED_HEAD:0);
     self->mArgs[self->mArgsNum] = NULL;
 
     return TRUE;
@@ -580,6 +579,125 @@ BOOL sCommand_add_redirect(sCommand* self, MANAGED char* name, BOOL env, BOOL qu
     return TRUE;
 }
 
+BOOL sCommand_add_command_without_command_name(sCommand* self, sCommand* command, char* sname, int sline)
+{
+    int i;
+    for(i=1; i<command->mArgsNum; i++) {
+        if(command->mArgsFlags[i] & XYZSH_ARGUMENT_ENV) {
+            sBuf buf;
+            memset(&buf, 0, sizeof(sBuf));
+            buf.mBuf = MALLOC(64);
+            buf.mBuf[0] = 0;
+            buf.mSize = 64;
+
+            char* p = command->mArgs[i];
+            while(*p) {
+                if(*p == PARSER_MAGIC_NUMBER_ENV) {
+                    p++;
+
+                    char buf2[128];
+                    char* p2 = buf2;
+                    while(*p != PARSER_MAGIC_NUMBER_ENV) {
+                        *p2++ = *p++;
+                    }
+                    p++;
+                    *p2 = 0;
+
+                    int num = atoi(buf2) + self->mEnvsNum;
+                    snprintf(buf2, 128, "%d", num);
+
+                    add_char_to_buf(&buf, PARSER_MAGIC_NUMBER_ENV);
+                    add_str_to_buf(&buf, buf2, strlen(buf2));
+                    add_char_to_buf(&buf, PARSER_MAGIC_NUMBER_ENV);
+                }
+                else {
+                    add_char_to_buf(&buf, *p++);
+                }
+            }
+            if(!sCommand_add_arg(self, MANAGED buf.mBuf, command->mArgsFlags[i] & XYZSH_ARGUMENT_ENV, command->mArgsFlags[i] & XYZSH_ARGUMENT_QUOTED, command->mArgsFlags[i] & XYZSH_ARGUMENT_QUOTED_HEAD, command->mArgsFlags[i] & XYZSH_ARGUMENT_GLOB, command->mArgsFlags[i] & XYZSH_ARGUMENT_OPTION, sname, sline))
+            {
+                return FALSE;
+            }
+        }
+        else {
+            char* arg = STRDUP(command->mArgs[i]);
+            if(!sCommand_add_arg(self, MANAGED arg, command->mArgsFlags[i] & XYZSH_ARGUMENT_ENV, command->mArgsFlags[i] & XYZSH_ARGUMENT_QUOTED, command->mArgsFlags[i] & XYZSH_ARGUMENT_QUOTED_HEAD, command->mArgsFlags[i] & XYZSH_ARGUMENT_GLOB, command->mArgsFlags[i] & XYZSH_ARGUMENT_OPTION, sname, sline))
+            {
+                return FALSE;
+            }
+        }
+    }
+    for(i=0; i<command->mEnvsNum; i++) {
+        sEnv* env = command->mEnvs + i;
+
+        if(env->mFlags & ENV_FLAGS_KIND_ENV) {
+            if(!sCommand_add_env(self, MANAGED STRDUP(env->mName), MANAGED STRDUP(env->mInitValue), MANAGED STRDUP(env->mKey), env->mFlags & ENV_FLAGS_KEY_ENV, env->mFlags & ENV_FLAGS_KEY_QUOTED_STRING, sname, sline))
+            {
+                return FALSE;
+            }
+        }
+        else {
+            sObject* block = block_clone_on_stack(env->mBlock, T_BLOCK);
+            if(!sCommand_add_env_block(self, block, env->mLineField, sname, sline)) {
+                return FALSE;
+            }
+        }
+    }
+    for(i=0; i<command->mBlocksNum; i++) {
+        sObject* block = block_clone_on_stack(command->mBlocks[i], T_BLOCK);
+        if(!sCommand_add_block(self, block, sname, sline)) {
+            return FALSE;
+        }
+    }
+    for(i=0; i<command->mRedirectsNum; i++) {
+        int flags = command->mRedirects[i];
+        if(flags & REDIRECT_ENV) {
+            sBuf buf;
+            memset(&buf, 0, sizeof(sBuf));
+            buf.mBuf = MALLOC(64);
+            buf.mBuf[0] = 0;
+            buf.mSize = 64;
+
+            char* p = command->mRedirectsFileNames[i];
+            while(*p) {
+                if(*p == PARSER_MAGIC_NUMBER_ENV) {
+                    p++;
+
+                    char buf2[128];
+                    char* p2 = buf2;
+                    while(*p != PARSER_MAGIC_NUMBER_ENV) {
+                        *p2++ = *p++;
+                    }
+                    p++;
+                    *p2 = 0;
+
+                    int num = atoi(buf2) + self->mEnvsNum;
+                    snprintf(buf2, 128, "%d", num);
+
+                    add_char_to_buf(&buf, PARSER_MAGIC_NUMBER_ENV);
+                    add_str_to_buf(&buf, buf2, strlen(buf2));
+                    add_char_to_buf(&buf, PARSER_MAGIC_NUMBER_ENV);
+                }
+                else {
+                    add_char_to_buf(&buf, *p++);
+                }
+            }
+            if(!sCommand_add_redirect(self, MANAGED buf.mBuf, flags & REDIRECT_ENV, flags & REDIRECT_QUOTED, flags & REDIRECT_GLOB, flags & REDIRECT_KIND, sname, sline)) 
+            {
+                return FALSE;
+            }
+        }
+        else {
+            if(!sCommand_add_redirect(self, MANAGED STRDUP(command->mRedirectsFileNames[i]), flags & REDIRECT_ENV, flags & REDIRECT_QUOTED, flags & REDIRECT_GLOB, flags & REDIRECT_KIND, sname, sline))
+            {
+                return FALSE;
+            }
+        }
+    }
+
+    return TRUE;
+}
+
 sStatment* sStatment_new(sObject* block, int sline, char* sname)
 {
     if(SBLOCK(block).mStatmentsNum >= SBLOCK(block).mStatmentsSize) {
@@ -750,6 +868,7 @@ void block_delete_on_stack(sObject* self)
     if(SBLOCK(self).mSource) FREE(SBLOCK(self).mSource);
 }
 
+
 int block_gc_children_mark(sObject* self)
 {
     int count = 0;
