@@ -22,7 +22,7 @@
 #include <ncurses/ncurses.h>
 #endif
 
-#include "xyzsh/xyzsh.h"
+#include "xyzsh.h"
 
 BOOL cmd_mshow(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
@@ -34,7 +34,7 @@ BOOL cmd_mshow(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         char buf[BUFSIZ];
         char*obj_kind[T_TYPE_MAX] = {
-            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "file dicriptor2", "job", "object", "function", "class", "external program", "completion", "external object"
+            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "file dicriptor2", "job", "object", "function", "class", "external program", "completion", "external object", "alias"
         };
         int size = snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[STYPE(object2)]);
 
@@ -255,6 +255,37 @@ BOOL cmd_objinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+BOOL cmd_typeof(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    if(runinfo->mArgsNumRuntime > 1) {
+        int i;
+        for(i=1; i<runinfo->mArgsNumRuntime; i++) {
+            sObject* object;
+            if(!get_object_from_argument(&object, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                return FALSE;
+            }
+
+            if(object) {
+                runinfo->mRCode = 0;
+
+                char buf[BUFSIZ];
+                char* obj_kind[T_TYPE_MAX] = {
+                    NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "file dicriptor2", "job", "object", "function", "class", "external program", "completion", "external object", "alias"
+                };
+                int size = snprintf(buf, BUFSIZ, "%s\n", obj_kind[STYPE(object)]);
+
+                if(!fd_write(nextout, buf, size)) {
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
+            }
+        }
+    }
+
+    return TRUE;
+}
+
 BOOL cmd_mrun(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     if(runinfo->mBlocksNum == 1) {
@@ -342,6 +373,7 @@ BOOL cmd_pwo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+/*
 BOOL cmd_co(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     if(runinfo->mArgsNumRuntime == 2) {
@@ -362,12 +394,24 @@ BOOL cmd_co(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
     return TRUE;
 }
+*/
 
 BOOL cmd_inherit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sObject* running_object = runinfo->mRunningObject;
     if(runinfo->mRunningObject) {
         switch(STYPE(running_object)) {
+        case T_ALIAS: {
+            sObject* parent = SALIAS(running_object).mParent;
+
+            if(parent) {
+                if(!run_object(parent, nextin, nextout, runinfo)) {
+                    return FALSE;
+                }
+            }
+            }
+            break;
+
         case T_FUN:  {
             sObject* parent = SFUN(running_object).mParent;
 
@@ -1405,6 +1449,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+
 BOOL cmd_object(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     if(sRunInfo_option(runinfo, "-new")) {
@@ -1529,6 +1574,99 @@ BOOL cmd_object(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+BOOL cmd_alias(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    if(runinfo->mArgsNumRuntime == 2) {
+        if(runinfo->mBlocksNum == 1) {
+            char* name = runinfo->mArgsRuntime[1];
+            sObject* block = runinfo->mBlocks[0];
+
+            if(sRunInfo_option(runinfo, "-local")) {
+                sObject* object = SFUN(runinfo->mRunningObject).mLocalObjects;
+                sObject* new_alias = ALIAS_NEW_GC(block, TRUE, NULL);
+                uobject_put(object, name, new_alias);
+            }
+            else {
+                sObject* object;
+                sObject* name2 = STRING_NEW_STACK("");
+                if(!get_object_prefix_and_name_from_argument(&object, name2, name, runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                    return FALSE;
+                }
+
+                sObject* parent = uobject_item(object, string_c_str(name2));
+                if(STYPE(parent) == T_ALIAS) parent = SALIAS(parent).mParent;
+
+                sObject* new_alias = ALIAS_NEW_GC(block, TRUE, parent);
+
+                uobject_put(object, string_c_str(name2), new_alias);
+            }
+        }
+        else {
+            /// input ///
+            if(runinfo->mFilter) {
+                char* name = runinfo->mArgsRuntime[1];
+                char* body = SFD(nextin).mBuf;
+
+                sObject* block = BLOCK_NEW_STACK();
+
+                int sline = 1;
+                if(!parse(body, "alias", &sline, block, NULL)) {
+                    return FALSE;
+                }
+
+                if(sRunInfo_option(runinfo, "-local")) {
+                    sObject* object = SFUN(runinfo->mRunningObject).mLocalObjects;
+                    sObject* new_alias = ALIAS_NEW_GC(block, TRUE, NULL);
+                    uobject_put(object, name, new_alias);
+                }
+                else {
+                    sObject* object;
+                    sObject* name2 = STRING_NEW_STACK("");
+                    if(!get_object_prefix_and_name_from_argument(&object, name2, name, runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                        return FALSE;
+                    }
+
+                    sObject* parent = uobject_item(object, string_c_str(name2));
+                    if(STYPE(parent) == T_ALIAS) parent = SALIAS(parent).mParent;
+                    sObject* new_alias = ALIAS_NEW_GC(block, TRUE, parent);
+                    uobject_put(object, string_c_str(name2), new_alias);
+                }
+
+                FREE(name);
+            }
+            /// output ///
+            else {
+                sObject* alias;
+                if(!get_object_from_argument(&alias, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                    return FALSE;
+                }
+
+                if(alias && STYPE(alias) == T_ALIAS) {
+                    sObject* block = SALIAS(alias).mBlock;
+                    char* source = SBLOCK(block).mSource;
+                    if(!fd_write(nextout, source, strlen(source))) {
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                        return FALSE;
+                    }
+                    if(!fd_write(nextout, "\n", 1)) {
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                        return FALSE;
+                    }
+                }
+                else {
+                    err_msg("not found variable", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    return FALSE;
+                }
+            }
+        }
+
+        runinfo->mRCode = 0;
+    }
+
+    return TRUE;
+}
 
 BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
@@ -1569,7 +1707,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                         char buf[128];
                         char* obj_kind[T_TYPE_MAX] = {
-                            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "file dicriptor", "job", "object", "function", "class", "external program", "completion", "external object"
+                            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "file dicriptor", "job", "object", "function", "class", "external program", "completion", "external object", "alias"
                         };
                         int size = snprintf(buf, 128, "%s\n", obj_kind[type]);
 
@@ -1640,7 +1778,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                 char buf[128];
                 char* obj_kind[T_TYPE_MAX] = {
-                    NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "file dicriptor", "job", "object", "function", "class", "external program", "completion", "external object"
+                    NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "file dicriptor", "job", "object", "function", "class", "external program", "completion", "external object", "alias"
                 };
                 int size = snprintf(buf, 128, "%s\n", obj_kind[type]);
 
