@@ -1,5 +1,5 @@
 #include "config.h"
-#include "xyzsh/xyzsh.h"
+#include "xyzsh.h"
 
 #if !defined(__CYGWIN__)
 #include <term.h>
@@ -363,19 +363,26 @@ static void editline_get_last_word(EditLine* el, sObject* last_word, BOOL* squot
             }
 
             if(found) {
-                string_put(last_word, "");
                 p++;
 
+                string_put(last_word, "");
+
                 if(*p == '\'') {
                     *squote = TRUE;
+                    *dquote = FALSE;
                     p++;
                     squote2 = TRUE;
                 }
                 else if(*p == '"') {
+                    *squote = FALSE;
                     *dquote = TRUE;
                     dquote2 = TRUE;
                     p++;
                 }
+                else {
+                    *squote = FALSE;
+                    *dquote = FALSE;
+                }
             }
             else {
                 string_push_back2(last_word, *p++);
@@ -959,16 +966,18 @@ static void my_endwin()
 
 static unsigned char my_getch()
 {
+    fd_set mask, read_ok;
+
+    FD_ZERO(&mask);
+    FD_SET(0, &mask);
+
     unsigned char key = 0;
     while(1) {
-        fd_set mask;
+        read_ok = mask;
 
-        FD_ZERO(&mask);
-        FD_SET(0, &mask);
+        select(1, &read_ok, NULL, NULL, NULL);
 
-        select(1, &mask, NULL, NULL, NULL);
-
-        if(FD_ISSET(0, &mask)) {
+        if(FD_ISSET(0, &read_ok)) {
             unsigned char buf[128];
             int n;
             if((n = read(0, buf, 1)) == 1) {
@@ -1777,592 +1786,6 @@ static el_action_t castamized_user_complete(EditLine *el, int ch)
     return res;
 }
 
-#define	EL_LEAVE	2
-
-static el_action_t ed_mynewline(EditLine *el, int ch)
-{
-    el_action_t res = 0;
-    stack_start_stack();
-
-    sObject* block = BLOCK_NEW_STACK();
-
-    const LineInfoW *lf = el_wline(el);
-
-    const int size = MB_LEN_MAX * (wcslen(lf->buffer) + 1);
-    char* buf = MALLOC(size);
-    wcstombs(buf, lf->buffer, size);
-
-    sObject* cmdline = STRING_NEW_STACK("");
-    string_push_back(cmdline, buf);
-    FREE(buf);
-
-//printf("cmdline (%s)\n", string_c_str(cmdline));
-
-    int sline = 1;
-    sObject* current_object = gCurrentObject;
-    BOOL result = parse(string_c_str(cmdline), "editline", &sline, block, &current_object);
-
-    /// in the block? get the block
-    if(!result && (SBLOCK(block).mCompletionFlags & (COMPLETION_FLAGS_BLOCK|COMPLETION_FLAGS_ENV_BLOCK))) {
-        el_winsertstr(el, L"\n");
-        res = CC_REFRESH;
-    }
-    else if(!result && (SBLOCK(block).mCompletionFlags & COMPLETION_FLAGS_HERE_DOCUMENT)) {
-        el_winsertstr(el, L"\n");
-        res = CC_REFRESH;
-    }
-    else {
-        res = ed_newline(el, 0);
-    }
-
-    stack_end_stack();
-
-    return res;
-}
-
-
-#define OKCMD    -1    // must be -1!
-
-private int read_getcmd(EditLine *el, el_action_t *cmdnum, Char *ch)
-{
-    el_action_t cmd;
-    int num;
-
-    el->el_errno = 0;
-    do {
-        if ((num = FUN(el,getc)(el, ch)) != 1) {// if EOF or error
-            el->el_errno = num == 0 ? 0 : errno;
-            return 0;    // not OKCMD
-        }
-
-#ifdef    KANJI
-        if ((*ch & 0200)) {
-            el->el_state.metanext = 0;
-            cmd = CcViMap[' '];
-            break;
-        } else
-#endif // KANJI
-
-        if (el->el_state.metanext) {
-            el->el_state.metanext = 0;
-            *ch |= 0200;
-        }
-#ifdef WIDECHAR
-        if (*ch >= N_KEYS)
-            cmd = ED_INSERT;
-        else
-#endif
-            cmd = el->el_map.current[(unsigned char) *ch];
-        if (cmd == ED_SEQUENCE_LEAD_IN) {
-            keymacro_value_t val;
-            switch (keymacro_get(el, ch, &val)) {
-            case XK_CMD:
-                cmd = val.cmd;
-                break;
-            case XK_STR:
-                FUN(el,push)(el, val.str);
-                break;
-#ifdef notyet
-            case XK_EXE:
-                // XXX: In the future to run a user function
-                RunCommand(val.str);
-                break;
-#endif
-            default:
-                EL_ABORT((el->el_errfile, "Bad XK_ type \n"));
-                break;
-            }
-        }
-        if (el->el_map.alt == NULL)
-            el->el_map.current = el->el_map.key;
-    } while (cmd == ED_SEQUENCE_LEAD_IN);
-    *cmdnum = cmd;
-    return OKCMD;
-}
-
-/*
-static int gHistorySugestionLines = -1;
-static int gHistorySugestionHeightOfCmdline;;
-
-static void history_sugestion_view_clear(EditLine* el)
-{
-    if(gHistorySugestionLines > 0) {
-        const int maxx = mgetmaxx();
-        char buf[1024];
-        int j;
-        for(j=0; j<maxx && j<1024; j++) {
-            buf[j] = ' ';
-        }
-        buf[j] = 0;
-
-        puts("");
-
-        for(j=0; j<gHistorySugestionLines; j++) {
-            printf("%s\n", buf);
-        }
-
-        /// back to previous position ///
-        const LineInfoW* lf = el_wline(el);
-        const int prompt_h = el->el_prompt.p_pos.h;
-        int height_of_cmdline = (prompt_h + wcswidth(lf->buffer, wcslen(lf->buffer))) / maxx + 1;
-
-        if(height_of_cmdline < gHistorySugestionHeightOfCmdline) {
-            height_of_cmdline = gHistorySugestionHeightOfCmdline;
-        }
-
-        int i;
-        for(i=0; i<gHistorySugestionLines + height_of_cmdline; i++) {
-            printf("\033[%dA", 1);
-            printf("\r\033[0K");
-        }
-
-        /// redraw prompt ///
-        el_wset(gEditLine, EL_REFRESH);
-
-        gHistorySugestionLines = -1;
-    }
-}
-
-static void hisotry_sugestion_view(EditLine* el)
-{
-    HistEventW ev;
-    (void)history_w(gHistory, &ev, H_FIRST);
-
-    const LineInfoW* li = el_wline(el);
-    
-    const wchar_t* histories[3] = { NULL, NULL, NULL};
-
-    char* env_line_num = getenv("XYZSH_HISTORY_SUGESTION_LINE_NUM");
-    int line_num;
-    if(env_line_num) {
-        line_num = atoi(env_line_num);
-    }
-    else {
-        line_num = 1;
-    }
-
-    int count = 0;
-    do {
-        if(wcsstr(ev.str, li->buffer)) {
-            histories[count] = ev.str;
-
-            count++;
-
-            if(count == line_num) {
-                break;
-            }
-        }
-    } while(history_w(gHistory, &ev, H_NEXT) == 0);
-
-    if(count > 0) {
-        /// view ///
-        puts("");
-
-        const int maxx = mgetmaxx();
-
-        int i;
-        for(i=0; i<count; i++) {
-            wchar_t buf[maxx/2+1];
-            memcpy(buf, histories[i], sizeof(wchar_t)*maxx/2);
-            buf[maxx/2] = 0;
-            wprintf(L"%ls\n", buf);
-        }
-
-        /// get height of cmdline ///
-        const LineInfoW* lf = el_wline(el);
-        const int prompt_h = el->el_prompt.p_pos.h;
-        const int height_of_cmdline = (prompt_h + wcswidth(lf->buffer, wcslen(lf->buffer))) / maxx + 1;
-
-        /// back to previous position ///
-        printf("\033[%dA", count + height_of_cmdline);
-
-        /// must be refresh cmdline ///
-        el_wset(gEditLine, EL_REFRESH);
-
-        gHistorySugestionLines = count;
-        gHistorySugestionHeightOfCmdline = height_of_cmdline;
-    }
-}
-*/
-
-public const Char * el_my_wgets(EditLine* el, int* nread, wchar_t* init_buffer, int cursor_pos)
-{
-    int retval;
-    el_action_t cmdnum = 0;
-    int num;        // how many chars we have read at NL
-    Char ch, *cp;
-    int crlf = 0;
-    int nrb;
-#ifdef FIONREAD
-    c_macro_t *ma = &el->el_chared.c_macro;
-#endif // FIONREAD
-
-// --- this is hack by ab25cq -- //
-// clear buffer 
-    const int sz = (size_t)(gEditLine->el_line.limit - gEditLine->el_line.buffer + EL_LEAVE);
-    (void) memset(gEditLine->el_line.buffer, 0, sz);
-    gEditLine->el_line.lastchar = gEditLine->el_line.cursor = gEditLine->el_line.buffer;
-// -- end --
-
-    if (nread == NULL)
-        nread = &nrb;
-    *nread = 0;
-
-    if (el->el_flags & NO_TTY) {
-        size_t idx;
-
-        cp = el->el_line.buffer;
-        while ((num = (*el->el_read.read_char)(el, cp)) == 1) {
-            
-            // make sure there is space for next character 
-            if (cp + 1 >= el->el_line.limit) {
-                idx = (size_t)(cp - el->el_line.buffer);
-                if (!ch_enlargebufs(el, (size_t)2))
-                    break;
-                cp = &el->el_line.buffer[idx];
-            }
-            cp++;
-            if (el->el_flags & UNBUFFERED)
-                break;
-            if (cp[-1] == '\r' || cp[-1] == '\n')
-                break;
-        }
-        if (num == -1) {
-            if (errno == EINTR)
-                cp = el->el_line.buffer;
-            el->el_errno = errno;
-        }
-
-        goto noedit;
-    }
-
-
-#ifdef FIONREAD
-    if (el->el_tty.t_mode == EX_IO && ma->level < 0) {
-        long chrs = 0;
-
-        (void) ioctl(el->el_infd, FIONREAD, &chrs);
-        if (chrs == 0) {
-            if (tty_rawmode(el) < 0) {
-                errno = 0;
-                *nread = 0;
-                return NULL;
-            }
-        }
-    }
-#endif // FIONREAD 
-
-    if ((el->el_flags & UNBUFFERED) == 0)
-        read_prepare(el);
-
-// -- this is hack by ab25cq ---
-if (init_buffer) {
-    el_winsertstr(el, init_buffer);
-
-    const LineInfoW* li = el_wline(el);
-
-    const int len = wcslen(li->buffer);
-
-    if(cursor_pos < 0) cursor_pos += len + 1;
-    if(cursor_pos < 0) cursor_pos = 0;
-    if(cursor_pos > len) cursor_pos = len;
-
-    el->el_line.cursor = el->el_line.buffer + cursor_pos;
-
-    re_refresh(el);
-}
-// --- end ---
-
-    if (el->el_flags & EDIT_DISABLED) {
-        size_t idx;
-
-        if ((el->el_flags & UNBUFFERED) == 0)
-            cp = el->el_line.buffer;
-        else
-            cp = el->el_line.lastchar;
-
-        terminal__flush(el);
-
-        while ((num = (*el->el_read.read_char)(el, cp)) == 1) {
-            // make sure there is space next character 
-            if (cp + 1 >= el->el_line.limit) {
-                idx = (size_t)(cp - el->el_line.buffer);
-                if (!ch_enlargebufs(el, (size_t)2))
-                    break;
-                cp = &el->el_line.buffer[idx];
-            }
-            cp++;
-            crlf = cp[-1] == '\r' || cp[-1] == '\n';
-            if (el->el_flags & UNBUFFERED)
-                break;
-            if (crlf)
-                break;
-        }
-
-        if (num == -1) {
-            if (errno == EINTR)
-                cp = el->el_line.buffer;
-            el->el_errno = errno;
-        }
-
-        goto noedit;
-    }
-
-// --- this is hack by ab25cq --- //
-    //hisotry_sugestion_view(el);
-// ----
-
-    for (num = OKCMD; num == OKCMD;) {    // while still editing this
-                         // line 
-#ifdef DEBUG_EDIT
-        read_debug(el);
-#endif // DEBUG_EDIT
-        // if EOF or error
-        if ((num = read_getcmd(el, &cmdnum, &ch)) != OKCMD) {
-            completion_view_clear(el, TRUE);
-            num = -1;
-
-// --- this is hack by ab25cq -- //
-// clear buffer for the time user pressed Control-C
-            const int sz = (size_t)(el->el_line.limit - el->el_line.buffer + EL_LEAVE);
-            (void) memset(el->el_line.buffer, 0, sz);
-            el->el_line.lastchar = el->el_line.cursor = el->el_line.buffer;
-// -- end --
-
-#ifdef DEBUG_READ
-            (void) fprintf(el->el_errfile,
-                "Returning from el_gets %d\n", num);
-#endif // DEBUG_READ 
-            break;
-        }
-
-/// --- this is hack by ab25cq --- ///
-        if(ch != '\t') {
-            gPressingTabTwice = FALSE;
-            completion_view_clear(el, FALSE);
-        }
-/// ----------------------------- ///
-
-        if (el->el_errno == EINTR) {
-            el->el_line.buffer[0] = '\0';
-            el->el_line.lastchar =
-                el->el_line.cursor = el->el_line.buffer;
-            break;
-        }
-        if ((unsigned int)cmdnum >= (unsigned int)el->el_map.nfunc) {    // BUG CHECK command 
-#ifdef DEBUG_EDIT
-            (void) fprintf(el->el_errfile,
-                "ERROR: illegal command from key 0%o\r\n", ch);
-#endif // DEBUG_EDIT 
-            continue;    // try again
-        }
-        // now do the real command
-#ifdef DEBUG_READ
-        {
-            el_bindings_t *b;
-            for (b = el->el_map.help; b->name; b++)
-                if (b->func == cmdnum)
-                    break;
-            if (b->name)
-                (void) fprintf(el->el_errfile,
-                    "Executing %s\n", b->name);
-            else
-                (void) fprintf(el->el_errfile,
-                    "Error command = %d\n", cmdnum);
-        }
-#endif // DEBUG_READ 
-        // vi redo needs these way down the levels... 
-        el->el_state.thiscmd = cmdnum;
-        el->el_state.thisch = ch;
-        if (el->el_map.type == MAP_VI &&
-            el->el_map.current == el->el_map.key &&
-            el->el_chared.c_redo.pos < el->el_chared.c_redo.lim) {
-            if (cmdnum == VI_DELETE_PREV_CHAR &&
-                el->el_chared.c_redo.pos != el->el_chared.c_redo.buf
-                && Isprint(el->el_chared.c_redo.pos[-1]))
-                el->el_chared.c_redo.pos--;
-            else
-                *el->el_chared.c_redo.pos++ = ch;
-        }
-        retval = (*el->el_map.func[cmdnum]) (el, ch);
-#ifdef DEBUG_READ
-        (void) fprintf(el->el_errfile,
-            "Returned state %d\n", retval );
-#endif // DEBUG_READ 
-
-        // save the last command here 
-        el->el_state.lastcmd = cmdnum;
-
-/*
-// -- this is hacked by ab25cq //
-        if(ch == '\n') {
-            history_sugestion_view_clear(el);
-            puts("");
-        }
-        else {
-            history_sugestion_view_clear(el);
-            hisotry_sugestion_view(el);
-        }
-// -------------------------------
-*/
-
-        // use any return value 
-        switch (retval) {
-        case CC_CURSOR:
-            re_refresh_cursor(el);
-            break;
-
-        case CC_REDISPLAY:
-            re_clear_lines(el);
-            re_clear_display(el);
-            // FALLTHROUGH 
-
-        case CC_REFRESH:
-            re_refresh(el);
-            break;
-
-        case CC_REFRESH_BEEP:
-            re_refresh(el);
-            terminal_beep(el);
-            break;
-
-        case CC_NORM:    // normal char 
-            break;
-
-        case CC_ARGHACK:    // Suggested by Rich Salz 
-            // <rsalz@pineapple.bbn.com> 
-            continue;    // keep going... 
-
-        case CC_EOF:    // end of file typed 
-            if ((el->el_flags & UNBUFFERED) == 0)
-                num = 0;
-            else if (num == -1) {
-                *el->el_line.lastchar++ = CONTROL('d');
-                el->el_line.cursor = el->el_line.lastchar;
-                num = 1;
-            }
-            break;
-
-        case CC_NEWLINE:    // normal end of line 
-            num = (int)(el->el_line.lastchar - el->el_line.buffer);
-            break;
-
-        case CC_FATAL:    // fatal error, reset to known state 
-#ifdef DEBUG_READ
-            (void) fprintf(el->el_errfile,
-                "*** editor fatal ERROR ***\r\n\n");
-#endif // DEBUG_READ 
-            // put (real) cursor in a known place 
-            re_clear_display(el);    // reset the display stuff 
-            ch_reset(el, 1);    // reset the input pointers 
-            re_refresh(el); // print the prompt again 
-            break;
-
-        case CC_ERROR:
-        default:    // functions we don't know about 
-#ifdef DEBUG_READ
-            (void) fprintf(el->el_errfile,
-                "*** editor ERROR ***\r\n\n");
-#endif // DEBUG_READ 
-            terminal_beep(el);
-            terminal__flush(el);
-            break;
-        }
-/// --- this is hack by ab25cq --- ///
-// visualize check for left and right paren ///
-        char* blink_matching_paren = getenv("XYZSH_BLINK_MATCHING_PAREN");
-        if(blink_matching_paren && (strcasecmp(blink_matching_paren, "on") == 0 || strcasecmp(blink_matching_paren, "true") == 0)) 
-        {
-            if(ch == ')') {
-                const int left_paren_pos_max = 128;
-                Char* lparen_pos = NULL;
-                Char* left_paren_pos[left_paren_pos_max];
-                memset(left_paren_pos, 0, sizeof(Char)*left_paren_pos_max);
-                int left_paren_count = 0;
-                Char* p = el->el_line.buffer;
-                while(p < el->el_line.cursor) {
-                    if(*p == '(') {
-                        if(left_paren_count < left_paren_pos_max) {
-                            left_paren_pos[left_paren_count++] = p;
-                        }
-                        p++;
-                    }
-                    else if(*p == ')') {
-                        left_paren_count--;
-                        if(left_paren_count < 0) {
-                            lparen_pos = NULL;
-                        }
-                        else {
-                            lparen_pos = left_paren_pos[left_paren_count];
-                        }
-                        p++;
-
-                    }
-                    else {
-                        p++;
-                    }
-                }
-
-                /// move cursor to previus ( point ///
-                if(lparen_pos) {
-                    const int prompt_h = el->el_prompt.p_pos.h;
-                    const int prompt_v = el->el_prompt.p_pos.v;
-
-                    const int len = lparen_pos - el->el_line.buffer;
-                    wchar_t* tmp = MALLOC(sizeof(wchar_t)* (len + 1));
-                    memcpy(tmp, el->el_line.buffer, sizeof(wchar_t)*len);
-                    tmp[len] = 0;
-
-                    const int term_len = wcswidth(tmp, wcslen(tmp));
-
-                    FREE(tmp);
-                    
-                    const int termsz = mgetmaxx();
-                    const int h = (term_len + prompt_h) % termsz;
-                    const int v = (term_len + prompt_h) / termsz;
-
-                    terminal_move_to_line(el, v);
-                    terminal_move_to_char(el, h);
-                    terminal__flush(el);
-                    
-                    /// sleep and move cursor back at previous point ///
-                    sleep(1);
-                    re_refresh_cursor(el);
-
-                }
-            }
-        }
-/// ---------------------------
-        el->el_state.argument = 1;
-        el->el_state.doingarg = 0;
-        el->el_chared.c_vcmd.action = NOP;
-        if (el->el_flags & UNBUFFERED)
-            break;
-    }
-
-    terminal__flush(el);        // flush any buffered output
-    // make sure the tty is set up correctly 
-    if ((el->el_flags & UNBUFFERED) == 0) {
-        read_finish(el);
-        *nread = num != -1 ? num : 0;
-    } else {
-        *nread = (int)(el->el_line.lastchar - el->el_line.buffer);
-    }
-    goto done;
-noedit:
-    el->el_line.cursor = el->el_line.lastchar = cp;
-    *cp = '\0';
-    *nread = (int)(el->el_line.cursor - el->el_line.buffer);
-done:
-    if (*nread == 0) {
-        if (num == -1) {
-            *nread = -1;
-            errno = el->el_errno;
-        }
-        return NULL;
-    } else
-        return el->el_line.buffer;
-}
 
 static void transform_lf_to_semicolon(wchar_t* buf, ALLOC wchar_t** result)
 {
@@ -2402,6 +1825,373 @@ static void transform_lf_to_semicolon(wchar_t* buf, ALLOC wchar_t** result)
     *p2 = 0;
 }
 
+#define OKCMD    -1    /* must be -1! */
+public int read_getcmd(EditLine *el, el_action_t *cmdnum, Char *ch);
+
+public const Char * el_my_wgets(EditLine* el, int* nread, wchar_t* init_buffer, int cursor_pos)
+{
+    int retval;
+    el_action_t cmdnum = 0;
+    int num;        /* how many chars we have read at NL */
+    Char ch, *cp;
+    int crlf = 0;
+    int nrb;
+#ifdef FIONREAD
+    c_macro_t *ma = &el->el_chared.c_macro;
+#endif /* FIONREAD */
+
+    if (nread == NULL)
+        nread = &nrb;
+    *nread = 0;
+
+    if (el->el_flags & NO_TTY) {
+        size_t idx;
+
+        cp = el->el_line.buffer;
+        while ((num = (*el->el_read.read_char)(el, cp)) == 1) {
+            /* make sure there is space for next character */
+            if (cp + 1 >= el->el_line.limit) {
+                idx = (size_t)(cp - el->el_line.buffer);
+                if (!ch_enlargebufs(el, (size_t)2))
+                    break;
+                cp = &el->el_line.buffer[idx];
+            }
+            cp++;
+            if (el->el_flags & UNBUFFERED)
+                break;
+            if (cp[-1] == '\r' || cp[-1] == '\n')
+                break;
+        }
+        if (num == -1) {
+            if (errno == EINTR)
+                cp = el->el_line.buffer;
+            el->el_errno = errno;
+        }
+
+        goto noedit;
+    }
+
+
+#ifdef FIONREAD
+    if (el->el_tty.t_mode == EX_IO && ma->level < 0) {
+        long chrs = 0;
+
+        (void) ioctl(el->el_infd, FIONREAD, &chrs);
+        if (chrs == 0) {
+            if (tty_rawmode(el) < 0) {
+                errno = 0;
+                *nread = 0;
+                return NULL;
+            }
+        }
+    }
+#endif /* FIONREAD */
+
+    if ((el->el_flags & UNBUFFERED) == 0) {
+        read_prepare(el);
+
+// -- this is hacked by ab25cq ---
+if (init_buffer) {
+el_winsertstr(el, init_buffer);
+
+const LineInfoW* li = el_wline(el);
+
+const int len = wcslen(li->buffer);
+
+if(cursor_pos < 0) cursor_pos += len + 1;
+if(cursor_pos < 0) cursor_pos = 0;
+if(cursor_pos > len) cursor_pos = len;
+
+el->el_line.cursor = el->el_line.buffer + cursor_pos;
+
+re_refresh(el);
+}
+// --- end ---
+    }
+
+    if (el->el_flags & EDIT_DISABLED) {
+        size_t idx;
+
+        if ((el->el_flags & UNBUFFERED) == 0)
+            cp = el->el_line.buffer;
+        else
+            cp = el->el_line.lastchar;
+
+        terminal__flush(el);
+
+        while ((num = (*el->el_read.read_char)(el, cp)) == 1) {
+            /* make sure there is space next character */
+            if (cp + 1 >= el->el_line.limit) {
+                idx = (size_t)(cp - el->el_line.buffer);
+                if (!ch_enlargebufs(el, (size_t)2))
+                    break;
+                cp = &el->el_line.buffer[idx];
+            }
+            cp++;
+            crlf = cp[-1] == '\r' || cp[-1] == '\n';
+            if (el->el_flags & UNBUFFERED)
+                break;
+            if (crlf)
+                break;
+        }
+
+        if (num == -1) {
+            if (errno == EINTR)
+                cp = el->el_line.buffer;
+            el->el_errno = errno;
+        }
+
+        goto noedit;
+    }
+
+    for (num = OKCMD; num == OKCMD;) {    /* while still editing this
+                         * line */
+#ifdef DEBUG_EDIT
+        read_debug(el);
+#endif /* DEBUG_EDIT */
+        /* if EOF or error */
+        if ((num = read_getcmd(el, &cmdnum, &ch)) != OKCMD) {
+            num = -1;
+
+// --- this is hacked by ab25cq -- //
+completion_view_clear(el, TRUE);
+// -- end --
+
+#ifdef DEBUG_READ
+            (void) fprintf(el->el_errfile,
+                "Returning from el_gets %d\n", num);
+#endif /* DEBUG_READ */
+            break;
+        }
+// --- this is hacked by ab25cq --- //
+if(ch != '\t') {
+gPressingTabTwice = FALSE;
+completion_view_clear(el, FALSE);
+}
+/// ----------------------------- ///
+
+        if (el->el_errno == EINTR) {
+            el->el_line.buffer[0] = '\0';
+            el->el_line.lastchar =
+                el->el_line.cursor = el->el_line.buffer;
+            break;
+        }
+        if ((unsigned int)cmdnum >= (unsigned int)el->el_map.nfunc) {    /* BUG CHECK command */
+#ifdef DEBUG_EDIT
+            (void) fprintf(el->el_errfile,
+                "ERROR: illegal command from key 0%o\r\n", ch);
+#endif /* DEBUG_EDIT */
+            continue;    /* try again */
+        }
+        /* now do the real command */
+#ifdef DEBUG_READ
+        {
+            el_bindings_t *b;
+            for (b = el->el_map.help; b->name; b++)
+                if (b->func == cmdnum)
+                    break;
+            if (b->name)
+                (void) fprintf(el->el_errfile,
+                    "Executing %s\n", b->name);
+            else
+                (void) fprintf(el->el_errfile,
+                    "Error command = %d\n", cmdnum);
+        }
+#endif /* DEBUG_READ */
+        /* vi redo needs these way down the levels... */
+        el->el_state.thiscmd = cmdnum;
+        el->el_state.thisch = ch;
+        if (el->el_map.type == MAP_VI &&
+            el->el_map.current == el->el_map.key &&
+            el->el_chared.c_redo.pos < el->el_chared.c_redo.lim) {
+            if (cmdnum == VI_DELETE_PREV_CHAR &&
+                el->el_chared.c_redo.pos != el->el_chared.c_redo.buf
+                && Isprint(el->el_chared.c_redo.pos[-1]))
+                el->el_chared.c_redo.pos--;
+            else
+                *el->el_chared.c_redo.pos++ = ch;
+        }
+        retval = (*el->el_map.func[cmdnum]) (el, ch);
+#ifdef DEBUG_READ
+        (void) fprintf(el->el_errfile,
+            "Returned state %d\n", retval );
+#endif /* DEBUG_READ */
+
+        /* save the last command here */
+        el->el_state.lastcmd = cmdnum;
+
+        /* use any return value */
+        switch (retval) {
+        case CC_CURSOR:
+            re_refresh_cursor(el);
+            break;
+
+        case CC_REDISPLAY:
+            re_clear_lines(el);
+            re_clear_display(el);
+            /* FALLTHROUGH */
+
+        case CC_REFRESH:
+            re_refresh(el);
+            break;
+
+        case CC_REFRESH_BEEP:
+            re_refresh(el);
+            terminal_beep(el);
+            break;
+
+        case CC_NORM:    /* normal char */
+            break;
+
+        case CC_ARGHACK:    /* Suggested by Rich Salz */
+            /* <rsalz@pineapple.bbn.com> */
+            continue;    /* keep going... */
+
+        case CC_EOF:    /* end of file typed */
+            if ((el->el_flags & UNBUFFERED) == 0)
+                num = 0;
+            else if (num == -1) {
+                *el->el_line.lastchar++ = CONTROL('d');
+                el->el_line.cursor = el->el_line.lastchar;
+                num = 1;
+            }
+            break;
+
+        case CC_NEWLINE:    /* normal end of line */
+            num = (int)(el->el_line.lastchar - el->el_line.buffer);
+            break;
+
+        case CC_FATAL:    /* fatal error, reset to known state */
+#ifdef DEBUG_READ
+            (void) fprintf(el->el_errfile,
+                "*** editor fatal ERROR ***\r\n\n");
+#endif /* DEBUG_READ */
+            /* put (real) cursor in a known place */
+            re_clear_display(el);    /* reset the display stuff */
+            ch_reset(el, 1);    /* reset the input pointers */
+            re_refresh(el); /* print the prompt again */
+            break;
+
+        case CC_ERROR:
+        default:    /* functions we don't know about */
+#ifdef DEBUG_READ
+            (void) fprintf(el->el_errfile,
+                "*** editor ERROR ***\r\n\n");
+#endif /* DEBUG_READ */
+            terminal_beep(el);
+            terminal__flush(el);
+            break;
+        }
+
+/// --- this is hacked by ab25cq --- ///
+// visualize checking for right paren ///
+char* blink_matching_paren = getenv("XYZSH_BLINK_MATCHING_PAREN");
+if(blink_matching_paren && (strcasecmp(blink_matching_paren, "on") == 0 || strcasecmp(blink_matching_paren, "true") == 0)) 
+{
+    if(ch == ')') {
+        const int left_paren_pos_max = 128;
+        Char* lparen_pos = NULL;
+        Char* left_paren_pos[left_paren_pos_max];
+        memset(left_paren_pos, 0, sizeof(Char)*left_paren_pos_max);
+        int left_paren_count = 0;
+        Char* p = el->el_line.buffer;
+        while(p < el->el_line.cursor) {
+            if(*p == '(') {
+                if(left_paren_count < left_paren_pos_max) {
+                    left_paren_pos[left_paren_count++] = p;
+                }
+                p++;
+            }
+            else if(*p == ')') {
+                left_paren_count--;
+                if(left_paren_count < 0) {
+                    lparen_pos = NULL;
+                }
+                else {
+                    lparen_pos = left_paren_pos[left_paren_count];
+                }
+                p++;
+
+            }
+            else {
+                p++;
+            }
+        }
+
+        /// move cursor to previus ( point ///
+        if(lparen_pos) {
+            const int prompt_h = el->el_prompt.p_pos.h;
+            const int prompt_v = el->el_prompt.p_pos.v;
+
+            const int len = lparen_pos - el->el_line.buffer;
+            wchar_t* tmp = MALLOC(sizeof(wchar_t)* (len + 1));
+            memcpy(tmp, el->el_line.buffer, sizeof(wchar_t)*len);
+            tmp[len] = 0;
+
+            const int term_len = wcswidth(tmp, wcslen(tmp));
+
+            FREE(tmp);
+            
+            const int termsz = mgetmaxx();
+            const int h = (term_len + prompt_h) % termsz;
+            const int v = (term_len + prompt_h) / termsz;
+
+            terminal_move_to_line(el, v);
+            terminal_move_to_char(el, h);
+            terminal__flush(el);
+            
+            /// sleep and move cursor back at previous point ///
+#if HAVE_NANOSLEEP
+            char* xyzsh_blink_matching_paren_milli = getenv("XYZSH_BLINK_MATCHING_PAREN_MILLI_SEC");
+            if(xyzsh_blink_matching_paren_milli) {
+                struct timespec req = { 0, 1000*1000*atoi(xyzsh_blink_matching_paren_milli) };
+
+                (void)nanosleep(&req, NULL);
+            }
+            else {
+                sleep(1);
+            }
+#else
+            sleep(1);
+#endif
+            re_refresh_cursor(el);
+
+        }
+    }
+}
+/// ---------------------------
+
+        el->el_state.argument = 1;
+        el->el_state.doingarg = 0;
+        el->el_chared.c_vcmd.action = NOP;
+        if (el->el_flags & UNBUFFERED)
+            break;
+    }
+
+    terminal__flush(el);        /* flush any buffered output */
+    /* make sure the tty is set up correctly */
+    if ((el->el_flags & UNBUFFERED) == 0) {
+        read_finish(el);
+        *nread = num != -1 ? num : 0;
+    } else {
+        *nread = (int)(el->el_line.lastchar - el->el_line.buffer);
+    }
+    goto done;
+noedit:
+    el->el_line.cursor = el->el_line.lastchar = cp;
+    *cp = '\0';
+    *nread = (int)(el->el_line.cursor - el->el_line.buffer);
+done:
+    if (*nread == 0) {
+        if (num == -1) {
+            *nread = -1;
+            errno = el->el_errno;
+        }
+        return NULL;
+    } else
+        return el->el_line.buffer;
+}
+
 static BOOL gEditlineEditing = FALSE;
 
 ALLOC char* editline(char* prompt, char* rprompt, char* text, int cursor_pos)
@@ -2409,12 +2199,6 @@ ALLOC char* editline(char* prompt, char* rprompt, char* text, int cursor_pos)
     if(gEditlineEditing) { // inhibit duplex activation
         return NULL;
     }
-
-    el_wset(gEditLine, EL_ADDFN, L"ed-complete", L"Complete argument", complete);
-    el_wset(gEditLine, EL_BIND, L"^I", L"ed-complete", NULL);
-    el_wset(gEditLine, EL_ADDFN, L"ed-cmdline-macro", L"Commandlien macro", cmdline_macro);
-    el_wset(gEditLine, EL_BIND, L"^X", L"ed-cmdline-macro", NULL);
-
     editline_signal();
 
     if(prompt == NULL) prompt = " > ";
@@ -2498,10 +2282,6 @@ ALLOC char* editline_user_castamized(char* prompt, char* rprompt, char* text, in
     }
 
     gUserCompleteBlock = block;
-    el_wset(gEditLine, EL_ADDFN, L"ed-complete", L"Complete argument", castamized_user_complete);
-    el_wset(gEditLine, EL_BIND, L"^I", L"ed-complete", NULL);
-    el_wset(gEditLine, EL_ADDFN, L"ed-cmdline-macro", L"Commandlien macro", cmdline_macro);
-    el_wset(gEditLine, EL_BIND, L"^X", L"ed-cmdline-macro", NULL);
 
     editline_signal();
 
@@ -2585,11 +2365,6 @@ ALLOC char* editline_no_completion(char* prompt, char* rprompt, char* text, int
         return NULL;
     }
 
-    el_wset(gEditLine, EL_ADDFN, L"ed-complete", L"Complete argument", null_complete);
-    el_wset(gEditLine, EL_BIND, L"^I", L"ed-complete", NULL);
-    el_wset(gEditLine, EL_ADDFN, L"ed-cmdline-macro", L"Commandlien macro", cmdline_macro);
-    el_wset(gEditLine, EL_BIND, L"^X", L"ed-cmdline-macro", NULL);
-
     editline_signal();
 
     if(prompt == NULL) prompt = " > ";
@@ -2846,6 +2621,169 @@ BOOL cmd_editline_line_buffer(sObject* nextin, sObject* nextout, sRunInfo* runin
     return TRUE;
 }
 
+#define    EL_LEAVE    2
+
+static el_action_t ed_my_newline(EditLine *el, int ch)
+{
+    el_action_t res = 0;
+    stack_start_stack();
+
+    sObject* block = BLOCK_NEW_STACK();
+
+    const LineInfoW *lf = el_wline(el);
+
+    const int size = MB_LEN_MAX * (wcslen(lf->buffer) + 1);
+    char* buf = MALLOC(size);
+    wcstombs(buf, lf->buffer, size);
+
+    sObject* cmdline = STRING_NEW_STACK("");
+    string_push_back(cmdline, buf);
+    FREE(buf);
+
+//printf("cmdline (%s)\n", string_c_str(cmdline));
+
+    int sline = 1;
+    sObject* current_object = gCurrentObject;
+    BOOL result = parse(string_c_str(cmdline), "editline", &sline, block, &current_object);
+
+    /// in the block? get the block
+    if(!result && (SBLOCK(block).mCompletionFlags & (COMPLETION_FLAGS_BLOCK|COMPLETION_FLAGS_ENV_BLOCK))) {
+        el_winsertstr(el, L"\n");
+        res = CC_REFRESH;
+    }
+    else if(!result && (SBLOCK(block).mCompletionFlags & COMPLETION_FLAGS_HERE_DOCUMENT)) {
+        el_winsertstr(el, L"\n");
+        res = CC_REFRESH;
+    }
+    else {
+        res = ed_newline(el, 0);
+    }
+
+    stack_end_stack();
+
+    return res;
+}
+
+static void editline_additional_keybind_vi()
+{
+    el_wset(gEditLine, EL_ADDFN, L"ed-complete", L"Complete argument", complete);
+    el_wset(gEditLine, EL_BIND, L"^I", L"ed-complete", NULL);
+    el_wset(gEditLine, EL_ADDFN, L"ed-cmdline-macro", L"Commandlien macro", cmdline_macro);
+    el_wset(gEditLine, EL_BIND, L"^X", L"ed-cmdline-macro", NULL);
+
+    el_wset(gEditLine, EL_ADDFN, L"ed-my-newline", L"Pressing Enter", ed_my_newline);
+    el_wset(gEditLine, EL_BIND, L"^J", L"ed-my-newline", NULL);
+    el_wset(gEditLine, EL_BIND, L"^M", L"ed-my-newline", NULL);
+}
+
+static void editline_additional_keybind_emacs()
+{
+    el_wset(gEditLine, EL_ADDFN, L"ed-complete", L"Complete argument", complete);
+    el_wset(gEditLine, EL_BIND, L"^I", L"ed-complete", NULL);
+    el_wset(gEditLine, EL_ADDFN, L"ed-cmdline-macro", L"Commandlien macro", cmdline_macro);
+    el_wset(gEditLine, EL_BIND, L"^X", L"ed-cmdline-macro", NULL);
+
+    el_wset(gEditLine, EL_ADDFN, L"ed-my-newline", L"Pressing Enter", ed_my_newline);
+    el_wset(gEditLine, EL_BIND, L"^J", L"ed-my-newline", NULL);
+    el_wset(gEditLine, EL_BIND, L"^M", L"ed-my-newline", NULL);
+}
+
+BOOL cmd_editline_keybind(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    char* mode;
+    
+    if(mode = sRunInfo_option_with_argument(runinfo, "-mode")) {
+        if(strcmp(mode, "vi") == 0) {
+            el_wset(gEditLine, EL_EDITOR, L"vi");
+            editline_additional_keybind_vi();
+            runinfo->mRCode = 0;
+        }
+        else if(strcmp(mode, "emacs") == 0) {
+            el_wset(gEditLine, EL_EDITOR, L"emacs");
+            editline_additional_keybind_emacs();
+            runinfo->mRCode = 0;
+        }
+    }
+    else if(runinfo->mArgsNumRuntime == 1) {
+        char* home = getenv("HOME");
+        if(home == NULL) {
+            err_msg("$HOME is NULL", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+            return FALSE;
+        }
+
+        char path[PATH_MAX];
+        snprintf(path, PATH_MAX, "%s/.xyzsh/tmpfile", home);
+        FILE* file = fopen(path, "w");
+        if(file == NULL) {
+            err_msg("can't ceate tmpfile", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+            return FALSE;
+        }
+
+        FILE* el_outfile = gEditLine->el_outfile;       // save prev el_outfile
+        gEditLine->el_outfile = file;
+        el_wset(gEditLine, EL_BIND, NULL, NULL, NULL);
+        gEditLine->el_outfile = el_outfile;
+        fclose(file);
+
+        sObject* str = STRING_NEW_STACK("");
+
+        int fd = open(path, O_RDONLY);
+
+        char buf[BUFSIZ];
+        while(1) {
+            int size = read(fd, buf, BUFSIZ);
+
+            if(size < 0) {
+                err_msg("can't read temporaly file", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                close(fd);
+                return FALSE;
+            }
+            else if(size == 0) {
+                break;
+            }
+            else {
+                if(!fd_write(nextout, buf, size)) {
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    fclose(file);
+                    return FALSE;
+                }
+            }
+        }
+
+        close(fd);
+
+        runinfo->mRCode = 0;
+    }
+    else if(runinfo->mArgsNumRuntime == 3) {
+        char* key = runinfo->mArgsRuntime[1];
+        char* func = runinfo->mArgsRuntime[2];
+
+        const int len = strlen(key) + 1;
+        wchar_t* wcskey = MALLOC(sizeof(wchar_t) * len);
+        mbstowcs(wcskey, key, len);
+
+        const int len2 = strlen(func) + 1;
+        wchar_t* wcsfunc = MALLOC(sizeof(wchar_t) * len2);
+        mbstowcs(wcsfunc, func, len2);
+        
+        if(el_wset(gEditLine, EL_BIND, wcskey, wcsfunc, NULL) < 0) {
+            err_msg("error on bind function to a key", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+
+            FREE(wcskey);
+            FREE(wcsfunc);
+            return FALSE;
+        }
+
+        FREE(wcskey);
+        FREE(wcsfunc);
+
+        runinfo->mRCode = 0;
+    }
+
+    return TRUE;
+}
+
 BOOL cmd_editline_inhibit_completion(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     if(gEditlineEditing && runinfo->mArgsNumRuntime == 2) {
@@ -3074,9 +3012,13 @@ void editline_object_init(sObject* self)
     uobject_put(editline, "point", NFUN_NEW_GC(cmd_editline_point, NULL, TRUE));
     uobject_put(editline, "replace_line", NFUN_NEW_GC(cmd_editline_replace_line, NULL, TRUE));
     uobject_put(editline, "forced_update_display", NFUN_NEW_GC(cmd_editline_forced_update_display, NULL, TRUE));
+
+    sObject* nfun = NFUN_NEW_GC(cmd_editline_keybind, NULL, TRUE);
+    (void)nfun_put_option_with_argument(nfun, STRDUP("-mode"));
+    uobject_put(editline, "keybind", nfun);
 }
 
-void xyzsh_editline_init()
+void xyzsh_editline_history_init()
 {
     /// set history size ///
     int history_size;
@@ -3119,24 +3061,27 @@ void xyzsh_editline_init()
     history_w(gHistory, &ev, H_SETSIZE, history_size);
     history_w(gHistory, &ev, H_LOAD, histfile);
 
+    el_wset(gEditLine, EL_HIST, history_w, gHistory);
+}
+
+void xyzsh_editline_init()
+{
     /// editline init ///
     gEditLine = el_init("xyzsh", stdin, stdout, stderr);
 
     el_wset(gEditLine, EL_EDITOR, L"emacs");
     el_wset(gEditLine, EL_SIGNAL, 1);
     el_wset(gEditLine, EL_PROMPT_ESC, editline_prompt, '\1');
-    el_wset(gEditLine, EL_HIST, history_w, gHistory);
-
-    el_wset(gEditLine, EL_BIND, L"^R", L"em-inc-search-prev", NULL);
-    el_wset(gEditLine, EL_ADDFN, L"ed-mynewline", L"Pressing Enter", ed_mynewline);
-    el_wset(gEditLine, EL_BIND, L"^J", L"ed-mynewline", NULL);
-    el_wset(gEditLine, EL_BIND, L"^M", L"ed-mynewline", NULL);
 
     el_source(gEditLine, NULL);
+
+    editline_additional_keybind_emacs();
 }
 
 void xyzsh_editline_final()
 {
+    //if(gHistorySuggestionTail) FREE(gHistorySuggestionTail);
+
     /// write history ///
     char* histfile = getenv("XYZSH_HISTFILE");
 
