@@ -1,6 +1,6 @@
 #include "config.h"
 
-#include "xyzsh/xyzsh.h"
+#include "xyzsh.h"
 #include <string.h>
 #include <stdio.h>
 #include <glob.h>
@@ -324,9 +324,9 @@ static BOOL expand_glob(sStrings* strings, char* str, sCommand* command, sObject
     return TRUE;
 }
 
-BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sRunInfo* runinfo)
+BOOL sCommand_expand_env(sCommand* command, sObject* object, sObject* nextin, sRunInfo* runinfo)
 {
-    BOOL external_command = fun == NULL || STYPE(fun) == T_EXTPROG;
+    const BOOL external_command = object == NULL || STYPE(object) == T_EXTPROG;
 
     /// command name ///
     char* arg = command->mArgs[0];
@@ -346,7 +346,7 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sRunI
             /// variable and glob ///
             if(command->mArgsFlags[i] & XYZSH_ARGUMENT_GLOB) {
                 sStrings* strings = sStrings_new();
-                if(!expand_parametor(strings, command->mArgs[i], command, fun, nextin, runinfo, command->mArgsFlags[i] & XYZSH_ARGUMENT_QUOTED)) {
+                if(!expand_parametor(strings, command->mArgs[i], command, object, nextin, runinfo, command->mArgsFlags[i] & XYZSH_ARGUMENT_QUOTED)) {
                     sString_delete(strings);
                     return FALSE;
                 }
@@ -356,7 +356,7 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sRunI
                     char* arg = strings->mStrings[j];
 
                     sStrings* strings2 = sStrings_new();
-                    if(!expand_glob(strings2, arg, command, fun, nextin, runinfo)) {
+                    if(!expand_glob(strings2, arg, command, object, nextin, runinfo)) {
                         sString_delete(strings);
                         sString_delete(strings2);
                         return FALSE;
@@ -375,7 +375,7 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sRunI
             /// variable and option ///
             else if(command->mArgsFlags[i] & XYZSH_ARGUMENT_OPTION) {
                 sStrings* strings = sStrings_new();
-                if(!expand_parametor(strings, command->mArgs[i], command, fun, nextin, runinfo, command->mArgsFlags[i] & XYZSH_ARGUMENT_QUOTED)) {
+                if(!expand_parametor(strings, command->mArgs[i], command, object, nextin, runinfo, command->mArgsFlags[i] & XYZSH_ARGUMENT_QUOTED)) {
                     sString_delete(strings);
                     return FALSE;
                 }
@@ -383,7 +383,7 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sRunI
                 int j;
                 for(j=0; j<strings->mNumber; j++) {
                     char* arg = strings->mStrings[j];
-                    if(fun && (STYPE(fun) == T_NFUN && nfun_option_with_argument(fun, arg) || STYPE(fun) == T_CLASS && class_option_with_argument(fun,arg) || STYPE(fun) == T_FUN && fun_option_with_argument(fun, arg) ))
+                    if(object && (STYPE(object) == T_NFUN && nfun_option_with_argument(object, arg) || STYPE(object) == T_CLASS && class_option_with_argument(object,arg) || STYPE(object) == T_FUN && fun_option_with_argument(object, arg) ))
                     {
                         if(j + 1 >= strings->mNumber) {
                             char buf[BUFSIZ];
@@ -424,7 +424,7 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sRunI
             /// variable ///
             else {
                 sStrings* strings = sStrings_new();
-                if(!expand_parametor(strings, command->mArgs[i], command, fun, nextin, runinfo, command->mArgsFlags[i] & XYZSH_ARGUMENT_QUOTED)) {
+                if(!expand_parametor(strings, command->mArgs[i], command, object, nextin, runinfo, command->mArgsFlags[i] & XYZSH_ARGUMENT_QUOTED)) {
                     sString_delete(strings);
                     return FALSE;
                 }
@@ -454,7 +454,7 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sRunI
             else {
                 char* arg = command->mArgs[i];
 
-                if(fun && (STYPE(fun) == T_NFUN && nfun_option_with_argument(fun, arg) || STYPE(fun) == T_CLASS && class_option_with_argument(fun,arg) || STYPE(fun) == T_FUN && fun_option_with_argument(fun, arg) ))
+                if(object && (STYPE(object) == T_NFUN && nfun_option_with_argument(object, arg) || STYPE(object) == T_CLASS && class_option_with_argument(object,arg) || STYPE(object) == T_FUN && fun_option_with_argument(object, arg) ))
                 {
                     if(i + 1 >= command->mArgsNum) {
                         char buf[BUFSIZ];
@@ -473,7 +473,7 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sRunI
 
                     if(command->mArgsFlags[i+1] & XYZSH_ARGUMENT_ENV) {
                         sStrings* strings = sStrings_new();
-                        if(!expand_parametor(strings, command->mArgs[i+1], command, fun, nextin, runinfo, command->mArgsFlags[i+1] & XYZSH_ARGUMENT_QUOTED)) {
+                        if(!expand_parametor(strings, command->mArgs[i+1], command, object, nextin, runinfo, command->mArgsFlags[i+1] & XYZSH_ARGUMENT_QUOTED)) {
                             sString_delete(strings);
                             return FALSE;
                         }
@@ -525,7 +525,7 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sRunI
         /// glob ///
         else if(command->mArgsFlags[i] & XYZSH_ARGUMENT_GLOB) {
             sStrings* strings = sStrings_new();
-            if(!expand_glob(strings, command->mArgs[i], command, fun, nextin, runinfo)) {
+            if(!expand_glob(strings, command->mArgs[i], command, object, nextin, runinfo)) {
                 sString_delete(strings);
                 return FALSE;
             }
@@ -563,7 +563,7 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sRunI
             if(command->mRedirects[i] & REDIRECT_ENV) {
                 if(command->mRedirects[i] & REDIRECT_GLOB) {
                     sStrings* strings = sStrings_new();
-                    if(!expand_parametor(strings, fname, command, fun, nextin, runinfo, command->mRedirects[i] & REDIRECT_QUOTED)) {
+                    if(!expand_parametor(strings, fname, command, object, nextin, runinfo, command->mRedirects[i] & REDIRECT_QUOTED)) {
                         sString_delete(strings);
                         return FALSE;
                     }
@@ -577,7 +577,7 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sRunI
                     char* fname2 = strings->mStrings[0];
 
                     sStrings* strings2 = sStrings_new();
-                    if(!expand_glob(strings2, fname2, command, fun, nextin, runinfo)) {
+                    if(!expand_glob(strings2, fname2, command, object, nextin, runinfo)) {
                         sString_delete(strings);
                         sString_delete(strings2);
                         return FALSE;
@@ -599,7 +599,7 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sRunI
                 }
                 else {
                     sStrings* strings = sStrings_new();
-                    if(!expand_parametor(strings, fname, command, fun, nextin, runinfo, command->mRedirects[i] & REDIRECT_QUOTED)) {
+                    if(!expand_parametor(strings, fname, command, object, nextin, runinfo, command->mRedirects[i] & REDIRECT_QUOTED)) {
                         sString_delete(strings);
                         return FALSE;
                     }
@@ -622,7 +622,7 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sRunI
             /// glob ///
             else if(command->mRedirects[i] & REDIRECT_GLOB) {
                 sStrings* strings = sStrings_new();
-                if(!expand_glob(strings, fname, command, fun, nextin, runinfo)) {
+                if(!expand_glob(strings, fname, command, object, nextin, runinfo)) {
                     sString_delete(strings);
                     return FALSE;
                 }
