@@ -15,7 +15,7 @@
 #include <dirent.h>
 #include <migemo.h>
 
-#include "xyzsh/xyzsh.h"
+#include "xyzsh.h"
 
 static migemo* gMigemo;
 static sObject* gMigemoCache;
