@@ -1,5 +1,5 @@
 #include "config.h"
-#include "xyzsh/xyzsh.h"
+#include "xyzsh.h"
 #include <string.h>
 #include <stdio.h>
 
