@@ -1,5 +1,5 @@
 #include "config.h"
-#include "xyzsh/xyzsh.h"
+#include "xyzsh.h"
 #include <errno.h>
 #include <stdlib.h>
 #include <fcntl.h>
@@ -49,6 +49,7 @@ typedef struct {
     BOOL mQuotedString;
     BOOL mMessagePassing;
     BOOL mOption;
+    BOOL mQuotedHead;
 } sBuf;
 
 static void add_char_to_buf(sBuf* buf, char c)
@@ -83,7 +84,7 @@ static BOOL read_backslash(char**p, sBuf* buf, char* sname, int* sline)
             return FALSE;
 
         case '\n':
-            add_char_to_buf(buf, '\n');
+            //add_char_to_buf(buf, '\n');
             (*sline)++;
             break;
 
@@ -109,6 +110,9 @@ static BOOL read_backslash(char**p, sBuf* buf, char* sname, int* sline)
             break;
 
         default:
+            if(buf->mLen == 0) {
+                buf->mQuotedHead = TRUE;
+            }
             add_char_to_buf(buf, **p);
     }
 
@@ -118,7 +122,7 @@ static BOOL read_backslash(char**p, sBuf* buf, char* sname, int* sline)
 }
 
 static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObject** current_object);
-static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCommand* command, sObject* block, sObject** current_object, BOOL read_until_bracket_close);
+static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCommand* command, sObject* block, sObject** current_object, BOOL read_until_bracket_close, BOOL read_until_backquote);
 
 #define XYZSH_VARIABLE_OBJECT_MAX 8
 
@@ -243,11 +247,11 @@ static BOOL read_dollar(char** p, sBuf* buf, char* sname, int* sline, sCommand*
         if(**p == '[') {
             (*p)++;
 
-            if(!read_one_argument(p, &key, sname, sline, command, block, current_object, TRUE)) 
+            if(!read_one_argument(p, &key, sname, sline, command, block, current_object, TRUE, FALSE)) 
             {
                 FREE(name.mBuf);
                 if(key.mEnv) {  // for readline
-                    (void)sCommand_add_arg(command, MANAGED key.mBuf, key.mEnv, key.mQuotedString, key.mGlob, key.mOption, sname, *sline);
+                    (void)sCommand_add_arg(command, MANAGED key.mBuf, key.mEnv, key.mQuotedString, key.mQuotedHead, key.mGlob, key.mOption, sname, *sline);
                 }
                 else {
                     FREE(key.mBuf);
@@ -283,9 +287,40 @@ static BOOL read_dollar(char** p, sBuf* buf, char* sname, int* sline, sCommand*
     return TRUE;
 }
 
+static BOOL block_parse_until_backquote(char** p, char* sname, int* sline, sObject* block, sObject** current_object);
+
+static BOOL read_backquote(char** p, sBuf* buf, char* sname, int* sline, sCommand* command, sObject* block, sObject** current_object)
+{
+    (*p)++;
+
+    eLineField lf = kLF;
+
+    sObject* block2 = BLOCK_NEW_STACK();
+    if(!block_parse_until_backquote(p, sname, sline, block2, current_object)) {
+        (void)sCommand_add_env_block(command, block2, lf, sname, *sline);
+        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_ENV_BLOCK;
+        SBLOCK(block).mCompletionFlags |= command->mEnvsNum & COMPLETION_FLAGS_BLOCK_OR_ENV_NUM;
+        return FALSE;
+    }
+
+    buf->mEnv = TRUE;
+
+    add_char_to_buf(buf, PARSER_MAGIC_NUMBER_ENV);
+    char buf2[128];
+    snprintf(buf2, 128, "%d", command->mEnvsNum);
+    add_str_to_buf(buf, buf2);
+    add_char_to_buf(buf, PARSER_MAGIC_NUMBER_ENV);
+
+    if(!sCommand_add_env_block(command, block2, lf, sname, *sline)) {
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
 static BOOL add_argument_to_command(MANAGED sBuf* buf, sCommand* command, char* sname, int* sline);
 
-static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCommand* command, sObject* block, sObject** current_object, BOOL read_until_bracket_close)
+static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCommand* command, sObject* block, sObject** current_object, BOOL read_until_bracket_close, BOOL read_until_backquote)
 {
     BOOL squote = FALSE;
     BOOL dquote = FALSE;
@@ -318,6 +353,11 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
                         return FALSE;
                     }
                 }
+                else if(**p == '`') {
+                    if(!read_backquote(p, buf, sname,sline, command, block, current_object)) {
+                        return FALSE;
+                    }
+                }
                 else {
                     if(**p == '\n') (*sline)++;
 
@@ -398,12 +438,32 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
                 }
             }
         }
+        /// backslash quote ///
+        else if(read_until_backquote && **p == '\\' && *(*p+1) =='`') {
+        }
         /// quote ///
         else if(**p == '\\') {
             if(!read_backslash(p, buf, sname, sline)) {
                 return FALSE;
             }
         }
+        // this must be previous position to glob 
+        else if(**p == '\n' || **p ==' ' || **p == '#' || **p == '&' || **p == '(' || **p == ')' || **p == ';' || **p =='|' || read_until_bracket_close && **p == ']') 
+        {
+            break;
+        }
+        /// read backquote
+        else if(**p == '`') {
+            if(read_until_backquote) {
+                break;
+            }
+            else {
+                if(!read_backquote(p, buf, sname,sline, command, block, current_object)) {
+                    return FALSE;
+                }
+            }
+        }
+        /// read dollar
         else if(**p == '$') {
             if(!read_dollar(p, buf, sname,sline, command, block, current_object)) {
                 return FALSE;
@@ -592,9 +652,6 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
 
             skip_spaces(p);
         }
-        else if(read_until_bracket_close && **p == ']') {
-            break;
-        }
         /// glob ///
         else if(command->mArgsNum > 0 && (**p == '[' || **p == ']' || **p == '?' || **p == '*')) {
             add_char_to_buf(buf, **p);
@@ -631,9 +688,6 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
             add_char_to_buf(buf, **p);
             (*p)++;
         }
-        else if(**p == '\n' || **p ==' ' || **p == '#' || **p == '&' || **p == '(' || **p == ')' || **p == ';' || **p == ']' || **p =='|') {
-            break;
-        }
         else if(**p == '/') {
             SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_ENV;
             add_char_to_buf(buf, **p);
@@ -712,7 +766,7 @@ void tilda_expasion(char* file, ALLOC char** file2)
     *file2 = STRDUP(result);
 }
 
-static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object);
+static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object, BOOL read_until_backquote);
 
 // result --> TRUE: success
 // result --> FALSE: There is a error message on gErrMsg. Please output the error message and stop running
@@ -730,7 +784,60 @@ static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObje
         /// go parsing ///
         sStatment* statment = sStatment_new(block, *sline, sname);
 
-        if(!read_statment(p, statment, block, sname, sline, current_object))
+        if(!read_statment(p, statment, block, sname, sline, current_object, FALSE))
+        {
+            return FALSE;
+        }
+
+        /// skip comment ///
+        if(**p == '#') skip_to_nextline(p, sline);
+
+        /// skip spaces at tail of statment ///
+        while(**p == ' ' || **p == '\t' || **p == '\n' && (*sline)++) { (*p)++; }
+
+        /// there is no command, delete last statment
+        if(statment->mCommandsNum == 0) {
+            sStatment_delete(statment);
+            memset(statment, 0, sizeof(sStatment));
+            SBLOCK(block).mStatmentsNum--;
+        }
+
+        if(**p == 0) {
+            err_msg("close block. require ) character before the end of statments", sname, *sline, NULL);
+            return FALSE;
+        }
+
+        SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_STATMENT_END;
+    }
+    (*p)++;
+
+    const int len = *p - source_head;
+    char* source = MALLOC(len);
+    memcpy(source, source_head, len-1);
+    source[len-1] = 0;
+
+    SBLOCK(block).mSource = source;
+
+    return TRUE;
+}
+
+// result --> TRUE: success
+// result --> FALSE: There is a error message on gErrMsg. Please output the error message and stop running
+static BOOL block_parse_until_backquote(char** p, char* sname, int* sline, sObject* block, sObject** current_object)
+{
+    char* source_head = *p;
+
+    while(**p != '`') {
+        /// skip spaces at head of statment ///
+        while(**p == ' ' || **p == '\t' || **p == '\n' && (*sline)++) { (*p)++; }
+
+        /// skip a comment ///
+        if(**p == '#') skip_to_nextline(p, sline);
+
+        /// go parsing ///
+        sStatment* statment = sStatment_new(block, *sline, sname);
+
+        if(!read_statment(p, statment, block, sname, sline, current_object, TRUE))
         {
             return FALSE;
         }
@@ -790,7 +897,7 @@ static BOOL add_argument_to_command(MANAGED sBuf* buf, sCommand* command, char*
     }
     /// add buf to command ///
     else if(buf->mQuotedString || buf->mBuf[0] != 0) {
-        if(!sCommand_add_arg(command, MANAGED buf->mBuf, buf->mEnv, buf->mQuotedString, buf->mGlob, buf->mOption, sname, *sline)) {
+        if(!sCommand_add_arg(command, MANAGED buf->mBuf, buf->mEnv, buf->mQuotedString, buf->mQuotedHead, buf->mGlob, buf->mOption, sname, *sline)) {
             return FALSE;
         }
     }
@@ -801,7 +908,7 @@ static BOOL add_argument_to_command(MANAGED sBuf* buf, sCommand* command, char*
     return TRUE;
 }
 
-static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object)
+static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object, BOOL read_until_backquote)
 {
     while(**p) {
         sBuf buf;
@@ -812,10 +919,10 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
 
         SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_AFTER_SPACE;;
 
-        if(!read_one_argument(p, &buf, sname, sline, command, block, current_object, FALSE)) 
+        if(!read_one_argument(p, &buf, sname, sline, command, block, current_object, FALSE, read_until_backquote)) 
         {
             if(buf.mEnv) {  // for readline
-                (void)sCommand_add_arg(command, MANAGED buf.mBuf, TRUE, buf.mQuotedString, buf.mGlob, buf.mOption, sname, *sline);
+                (void)sCommand_add_arg(command, MANAGED buf.mBuf, TRUE, buf.mQuotedString, buf.mQuotedHead, buf.mGlob, buf.mOption, sname, *sline);
             }
             else {
                 FREE(buf.mBuf);
@@ -848,7 +955,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
                     return FALSE;
                 }
 
-                if(!sCommand_add_arg(command, MANAGED STRDUP("subshell"), 0, 0, 0, 0, sname, *sline)) {
+                if(!sCommand_add_arg(command, MANAGED STRDUP("subshell"), 0, 0, 0, 0, 0 , sname, *sline)) {
                     return FALSE;
                 }
                 if(!sCommand_add_block(command, block2, sname, *sline)) {
@@ -919,7 +1026,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
 /// read statment
 /// TRUE: OK
 /// FALSE: indicate to occure error
-static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object)
+static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object, BOOL read_until_backquote)
 {
     /// a reversing code ///
     if(**p == '!') {
@@ -968,7 +1075,7 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
         SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_INPUTING_COMMAND_NAME;
 
         sCommand* command = sCommand_new(statment);
-        if(!read_command(p, command, statment, block, sname, sline, current_object)) {
+        if(!read_command(p, command, statment, block, sname, sline, current_object, read_until_backquote)) {
             return FALSE;
         }
 
@@ -1064,7 +1171,7 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
     else if(**p == 0) {
         statment->mFlags |= STATMENT_FLAGS_NORMAL;
     }
-    else if(**p == ')' || **p == '#') {
+    else if(**p == ')' || **p == '#' || read_until_backquote && **p == '`') {
     }
     else {
         char buf[128];
@@ -1092,7 +1199,7 @@ BOOL parse(char* p, char* sname, int* sline, sObject* block, sObject** current_o
         /// go parsing ///
         sStatment* statment = sStatment_new(block, *sline, sname);
 
-        if(!read_statment(&p, statment, block, sname, sline, current_object))
+        if(!read_statment(&p, statment, block, sname, sline, current_object, FALSE))
         {
             return FALSE;
         }
