@@ -1,5 +1,5 @@
 #include "config.h"
-#include "xyzsh/xyzsh.h"
+#include "xyzsh.h"
 #include <string.h>
 #include <stdio.h>
 #include <signal.h>
@@ -93,7 +93,7 @@ static void xyzsh_read_rc_mini()
 
 void xyzsh_init(enum eAppType app_type, BOOL no_runtime_script)
 {
-    setenv("XYZSH_VERSION", "1.4.3", 1);
+    setenv("XYZSH_VERSION", "1.4.4", 1);
     setenv("XYZSH_DOCDIR", DOCDIR, 1);
     setenv("XYZSH_DATAROOTDIR", DOCDIR, 1);
     setenv("XYZSH_EXT_PATH", EXTDIR, 1);
@@ -151,13 +151,14 @@ void xyzsh_init(enum eAppType app_type, BOOL no_runtime_script)
     gc_init(1);
     run_init(app_type);
     load_init();
+    xyzsh_editline_init();
 
     gDirStack = VECTOR_NEW_GC(10, FALSE);
     uobject_put(gXyzshObject, "_dir_stack", gDirStack);
 
     char* term_env = getenv("TERM");
     if(term_env != NULL && strcmp(term_env, "") != 0) {
-        mcurses_init(kTKUtf8);
+        mcurses_init();
     }
 
     if(!xyzsh_rehash("init", 0)) {
@@ -174,7 +175,7 @@ void xyzsh_init(enum eAppType app_type, BOOL no_runtime_script)
         xyzsh_read_rc_mini();
     }
 
-    xyzsh_editline_init(); // must be after runtime script
+    xyzsh_editline_history_init(); // must be after runtime script
 }
 
 void xyzsh_final()
@@ -536,17 +537,3 @@ void xyzsh_set_signal_optc()
 
     if(xyzsh_set_signal_other) xyzsh_set_signal_other();
 }
-
-char* xstrncpy(char* des, char* src, int size)
-{
-    des[size-1] = 0;
-    return strncpy(des, src, size-1);
-}
-
-char* xstrncat(char* des, char* str, int size)
-{
-    des[size-1] = 0;
-    return strncat(des, str, size-1);
-}
-
-
